/*
TCP Client/Server Performance Test Application
Copyright (C) 2005-2014, James Gibbons
Released under the GNU Lesser General Public License version 2.1
See http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
*/
#ifndef QT_TCP_CLIENTSERVERAPP_H
#define QT_TCP_CLIENTSERVERAPP_H

#include <QMainWindow>
#include <QTimer>
#include <QtNetwork>

namespace Ui {
	class Qt_TCP_ClientServerAppClass;
}

//----------------------------------------------------------------------------
// constants

// 32K to 64K is optimal (Win32 limits I/O records to 64K before buffering)

static const int BUF_SIZE=1024*32;

// system states
enum tNetStatus {NET_RESET=0,NET_SOCK_SERVER,NET_SOCK_CLIENT};

#if defined(Q_OS_WIN32) || defined(Q_OS_WIN64)

// define minimum Windows support for NT

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0400
#endif

#ifndef WINVER
#define WINVER _WIN32_WINNT
#endif

#include <windows.h>

//---------------------------------------------------------------------------
// high resolution timer object with inline functions
class THighResTimer
{
private:
	LARGE_INTEGER t0,t1,f;
	double et,last_dt;
	bool na;
protected:
public:
	THighResTimer(void)
	{
		QueryPerformanceFrequency(&f);
		na=f.QuadPart==0;
		HighResReset();
	}

	void HighResReset(void)   // reset timer
	{
		QueryPerformanceCounter(&t0);
		last_dt=0.0;
		et=0.0;
	}

	double HighResTime(void)  // elapsed time in seconds
	{
		double dt;

		if (na) return 0.0;

		QueryPerformanceCounter(&t1);
		dt=t1.QuadPart-t0.QuadPart;
		if (dt<0.0)
		{
			et=et+last_dt/f.QuadPart;
			t0=t1;
			last_dt=0.0;
			return et;
		}
		else
		{
			last_dt=dt;
			return et+dt/f.QuadPart;
		}
	}
};

#endif

#if defined(Q_OS_LINUX) || defined(Q_OS_MAC)

#include <sys/time.h>

//---------------------------------------------------------------------------
// high resolution timer object with inline functions
// only as good as the Linux gettimeofday function (possibly micro-second precision)
class THighResTimer
{
private:
	double sec1;
protected:
public:
	void HighResReset(void)   // reset timer
	{
		struct timeval tv1;
		gettimeofday(&tv1,NULL);
		sec1=tv1.tv_sec+tv1.tv_usec/1000000.0;
	}

	THighResTimer(void)
	{
		HighResReset();
	}

	double HighResTime(void)  // elapsed time in seconds
	{
		struct timeval tv2;
		gettimeofday(&tv2,NULL);
		return (tv2.tv_sec+tv2.tv_usec/1000000.0)-sec1;
	}
};

#endif

//---------------------------------------------------------------------------

class Qt_TCP_ClientServerApp : public QMainWindow
{
	Q_OBJECT

public:
	explicit Qt_TCP_ClientServerApp(QWidget *parent = 0);
	~Qt_TCP_ClientServerApp();

private:
	Ui::Qt_TCP_ClientServerAppClass *ui;

	THighResTimer tHRT,rHRT;	// transmit and receive timers
	QTimer runTimer;
	QTcpServer *theServer;
	QTcpSocket *serverSocket;
	QTcpSocket *clientSocket;
	QUdpSocket *udpSocket;

	tNetStatus NetStatus;
	quint64 NSend,NRec,TSend,TRec;

	char *IOBuffer;
	int SizeIOBuffer;
	char CheckBuf;
	QString UDPString;
	char UDPSig,UDPCheckSig;
	unsigned int UDPSigErrors;

	void SetButtons(bool bState);
	void NewIOBuffer(void);
	void ReleaseIOBuffer(void);
	void AddMessage(QString s);

private slots:
	void on_sendData_clicked(bool);
	void on_clientButton_clicked();
	void on_serverButton_clicked();
	void on_stopButton_clicked();
	void runTimerTick(void);
	void serverConnect(void);
	void serverRead(void);
	void serverDisconnect(void);
	void serverError(QAbstractSocket::SocketError socketError);
	void clientConnect(void);
	void clientRead(void);
	void clientDisconnect(void);
	void clientError(QAbstractSocket::SocketError socketError);
	void udpRead(void);
	void udpError(QAbstractSocket::SocketError socketError);
};

#endif // QT_TCP_CLIENTSERVERAPP_H
